from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING, Iterator, List, Optional

from root.generated.openapi_client.api.objectives_api import ObjectivesApi
from root.generated.openapi_client.models.objective_list import ObjectiveList
from root.generated.openapi_client.models.objective_request import ObjectiveRequest

from .generated.openapi_client import ApiClient
from .skills import Skills
from .utils import iterate_cursor_list

if TYPE_CHECKING:
    from .validators import Validator


class Objectives:
    """Objectives (sub) API

    Note:

      The construction of the API instance should be handled by
      accesing an attribute of a :class:`root.client.RootSignals` instance.
    """

    def __init__(self, client: ApiClient):
        self.client = client

    def create(self, *, intent: Optional[str] = None, validators: Optional[List[Validator]] = None) -> str:
        """Create a new objective and return its ID.

        Args:

          intent: The intent of the objective.

          validators: An optional list of validators.

        """
        skills = Skills(self.client)
        request = ObjectiveRequest(
            intent=intent,
            validators=[validator._to_request(skills) for validator in validators or []],
        )
        api_instance = ObjectivesApi(self.client)
        return api_instance.objectives_create(objective_request=request).id

    def delete(self, objective_id: str) -> None:
        """
        Delete the objective from the registry.

        Args:

          objective_id: The objective to be deleted.

        """
        api_instance = ObjectivesApi(self.client)
        return api_instance.objectives_destroy(id=objective_id)

    def list(self, *, intent: Optional[str] = None, limit: int = 100) -> Iterator[ObjectiveList]:
        """Iterate through the objectives.

        Note:

          The call will list only publicly available global objectives and
          those objectives available to the organzation(s) of the user.

        Args:
          intent: Specific intent the returned objectives must match.
          limit: Number of entries to iterate through at most.

        """
        api_instance = ObjectivesApi(self.client)
        yield from iterate_cursor_list(partial(api_instance.objectives_list, intent=intent), limit=limit)



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing_extensions import Annotated, Self

from root.generated.openapi_client.models.data_loader_request import DataLoaderRequest
from root.generated.openapi_client.models.evaluator_demonstrations_request import EvaluatorDemonstrationsRequest
from root.generated.openapi_client.models.input_variable_request import InputVariableRequest
from root.generated.openapi_client.models.model_params_request import ModelParamsRequest
from root.generated.openapi_client.models.objective_request import ObjectiveRequest
from root.generated.openapi_client.models.reference_variable_request import ReferenceVariableRequest
from root.generated.openapi_client.models.status_enum import StatusEnum


class SkillRequest(BaseModel):
    """
    SkillRequest
    """  # noqa: E501

    change_note: Optional[StrictStr] = None
    data_loaders: Optional[List[DataLoaderRequest]] = None
    evaluator_demonstrations: Optional[List[EvaluatorDemonstrationsRequest]] = None
    evaluator_only_offline: Optional[StrictBool] = Field(
        default=None, description="Do not run the evaluator if used as an Objective validator."
    )
    evaluator_require_reference_variables: Optional[StrictBool] = None
    input_variables: Optional[List[InputVariableRequest]] = None
    is_evaluator: Optional[StrictBool] = None
    model_params: Optional[ModelParamsRequest] = None
    models: Optional[List[Annotated[str, Field(min_length=1, strict=True)]]] = None
    name: Optional[Annotated[str, Field(min_length=1, strict=True, max_length=1000)]] = None
    objective: Optional[ObjectiveRequest] = None
    objective_id: Optional[StrictStr] = None
    pii_filter: Optional[StrictBool] = Field(
        default=None, description="Filter out personally identifiable information before sending a prompt to a LLM"
    )
    prompt: Optional[StrictStr] = None
    reference_variables: Optional[List[ReferenceVariableRequest]] = None
    status: Optional[StatusEnum] = None
    system_message: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "change_note",
        "data_loaders",
        "evaluator_demonstrations",
        "evaluator_only_offline",
        "evaluator_require_reference_variables",
        "input_variables",
        "is_evaluator",
        "model_params",
        "models",
        "name",
        "objective",
        "objective_id",
        "pii_filter",
        "prompt",
        "reference_variables",
        "status",
        "system_message",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SkillRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in data_loaders (list)
        _items = []
        if self.data_loaders:
            for _item in self.data_loaders:
                if _item:
                    _items.append(_item.to_dict())
            _dict["data_loaders"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in evaluator_demonstrations (list)
        _items = []
        if self.evaluator_demonstrations:
            for _item in self.evaluator_demonstrations:
                if _item:
                    _items.append(_item.to_dict())
            _dict["evaluator_demonstrations"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in input_variables (list)
        _items = []
        if self.input_variables:
            for _item in self.input_variables:
                if _item:
                    _items.append(_item.to_dict())
            _dict["input_variables"] = _items
        # override the default output from pydantic by calling `to_dict()` of model_params
        if self.model_params:
            _dict["model_params"] = self.model_params.to_dict()
        # override the default output from pydantic by calling `to_dict()` of objective
        if self.objective:
            _dict["objective"] = self.objective.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in reference_variables (list)
        _items = []
        if self.reference_variables:
            for _item in self.reference_variables:
                if _item:
                    _items.append(_item.to_dict())
            _dict["reference_variables"] = _items
        # set to None if change_note (nullable) is None
        # and model_fields_set contains the field
        if self.change_note is None and "change_note" in self.model_fields_set:
            _dict["change_note"] = None

        # set to None if evaluator_demonstrations (nullable) is None
        # and model_fields_set contains the field
        if self.evaluator_demonstrations is None and "evaluator_demonstrations" in self.model_fields_set:
            _dict["evaluator_demonstrations"] = None

        # set to None if model_params (nullable) is None
        # and model_fields_set contains the field
        if self.model_params is None and "model_params" in self.model_fields_set:
            _dict["model_params"] = None

        # set to None if objective (nullable) is None
        # and model_fields_set contains the field
        if self.objective is None and "objective" in self.model_fields_set:
            _dict["objective"] = None

        # set to None if objective_id (nullable) is None
        # and model_fields_set contains the field
        if self.objective_id is None and "objective_id" in self.model_fields_set:
            _dict["objective_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SkillRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "change_note": obj.get("change_note"),
                "data_loaders": [DataLoaderRequest.from_dict(_item) for _item in obj["data_loaders"]]
                if obj.get("data_loaders") is not None
                else None,
                "evaluator_demonstrations": [
                    EvaluatorDemonstrationsRequest.from_dict(_item) for _item in obj["evaluator_demonstrations"]
                ]
                if obj.get("evaluator_demonstrations") is not None
                else None,
                "evaluator_only_offline": obj.get("evaluator_only_offline"),
                "evaluator_require_reference_variables": obj.get("evaluator_require_reference_variables"),
                "input_variables": [InputVariableRequest.from_dict(_item) for _item in obj["input_variables"]]
                if obj.get("input_variables") is not None
                else None,
                "is_evaluator": obj.get("is_evaluator"),
                "model_params": ModelParamsRequest.from_dict(obj["model_params"])
                if obj.get("model_params") is not None
                else None,
                "models": obj.get("models"),
                "name": obj.get("name"),
                "objective": ObjectiveRequest.from_dict(obj["objective"]) if obj.get("objective") is not None else None,
                "objective_id": obj.get("objective_id"),
                "pii_filter": obj.get("pii_filter"),
                "prompt": obj.get("prompt"),
                "reference_variables": [
                    ReferenceVariableRequest.from_dict(_item) for _item in obj["reference_variables"]
                ]
                if obj.get("reference_variables") is not None
                else None,
                "status": obj.get("status"),
                "system_message": obj.get("system_message"),
            }
        )
        return _obj



"""Root Signals SDK.

The use should start by creating a :class:`root.client.RootSignals` instance.

Example::

  from root import RootSignals
  client = RootSignals()

"""

from .__about__ import __version__
from .client import RootSignals

# Note: PEP-396 was rejected but we provide __version__ anyway
# ( https://peps.python.org/pep-0396/ )
__all__ = ["__version__", "RootSignals"]



from root import RootSignals
from root.validators import Validator

client = RootSignals()

cooking_skill = client.skills.create(
    prompt="Find me a good recipe for Italian food.",
    validators=[
        Validator(
            evaluator_name="Cooking recipe evaluator",
            prompt="Is it a cooking recipe",
            threshold=0.1,
        ),
        Validator(
            evaluator_name="Truthfulness",
            threshold=0.5,
        ),
    ],
)
response = cooking_skill.run()

# Check if the recipe was about cooking
print(response.validation)

# {'validator_results': [{'evaluator_name': 'Cooking recipe
# evaluator', 'evaluator_id': ..., 'threshold': 0.1, 'is_valid': True,
# 'result': 0.8, 'status': 'finished'}], 'is_valid': True}



# coding: utf-8

"""
Root Signals API

Root Signals JSON API provides a way to access Root Signals using provisioned API token

The version of the OpenAPI document: 1.0.0 (latest)
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing_extensions import Annotated, Self


class ModelCreate(BaseModel):
    """
    ModelCreate
    """  # noqa: E501

    default_key: Optional[Annotated[str, Field(strict=True, max_length=4000)]] = None
    id: StrictStr
    is_local: Optional[StrictBool] = None
    max_output_token_count: Optional[StrictInt] = None
    max_token_count: Optional[Annotated[int, Field(le=2147483647, strict=True, ge=-2147483648)]] = None
    model: Optional[StrictStr] = None
    name: Annotated[str, Field(strict=True, max_length=100)]
    url: Optional[Annotated[str, Field(strict=True, max_length=1024)]] = None
    __properties: ClassVar[List[str]] = [
        "default_key",
        "id",
        "is_local",
        "max_output_token_count",
        "max_token_count",
        "model",
        "name",
        "url",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModelCreate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set(
            [
                "id",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if default_key (nullable) is None
        # and model_fields_set contains the field
        if self.default_key is None and "default_key" in self.model_fields_set:
            _dict["default_key"] = None

        # set to None if max_token_count (nullable) is None
        # and model_fields_set contains the field
        if self.max_token_count is None and "max_token_count" in self.model_fields_set:
            _dict["max_token_count"] = None

        # set to None if url (nullable) is None
        # and model_fields_set contains the field
        if self.url is None and "url" in self.model_fields_set:
            _dict["url"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModelCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "default_key": obj.get("default_key"),
                "id": obj.get("id"),
                "is_local": obj.get("is_local"),
                "max_output_token_count": obj.get("max_output_token_count"),
                "max_token_count": obj.get("max_token_count"),
                "model": obj.get("model"),
                "name": obj.get("name"),
                "url": obj.get("url"),
            }
        )
        return _obj
